@@ -3,7 +3,7 @@
                              -------------------
     begin                : Wed Feb 20 2002
     copyright            : (C) 2002 by Tue and Ken Haste Andersen
-    email                : 
+    email                :
  ***************************************************************************/
 
 /***************************************************************************
@@ -18,26 +18,27 @@
 #ifndef CONTROLPUSHBUTTON_H
 #define CONTROLPUSHBUTTON_H
 
+#include <Q3Action>
 #include "controlobject.h"
 #include "defs.h"
-#include <qaction.h>
 
 /**
   *@author Tue and Ken Haste Andersen
   */
-  
+
 class ControlPushButton : public ControlObject
 {
     Q_OBJECT
 public:
-    ControlPushButton(ConfigKey key, bool bMidiSimulateLatching=false);
+    ControlPushButton(ConfigKey key);
     ~ControlPushButton();
+    void setToggleButton(bool bIsToggleButton);
 
 protected:
-    void setValueFromMidi(MidiCategory c, int v);
+    void setValueFromMidi(MidiCategory c, double v);
 
 private:
-    bool m_bMidiSimulateLatching;
+    bool m_bIsToggleButton;
 };
 
 #endif
