@@ -28,8 +28,8 @@
 #include "controlobjectthreadmain.h"
 #include "mixxxview.h"
 #include "widget/wnumberpos.h"
-#include "widget/wnumberbpm.h"
 #include "engine/enginebuffer.h"
+#include "engine/ratecontrol.h"
 
 DlgPrefControls::DlgPrefControls(QWidget * parent, MixxxView * pView, MixxxApp * mixxx, ConfigObject<ConfigValue> * pConfig) :  QWidget(parent), Ui::DlgPrefControlsDlg()
 {
@@ -61,18 +61,12 @@ DlgPrefControls::DlgPrefControls(QWidget * parent, MixxxView * pView, MixxxApp *
         m_pConfig->set(ConfigKey("[Controls]","PositionDisplay"),ConfigValue(0));
     if (m_pConfig->getValueString(ConfigKey("[Controls]","PositionDisplay")).toInt() == 1)
     {
-        if (pView->m_pNumberPosCh1)
-            pView->m_pNumberPosCh1->setRemain(true);
-        if (pView->m_pNumberPosCh2)
-            pView->m_pNumberPosCh2->setRemain(true);
+        pView->slotSetDurationRemaining(true);
         ComboBoxPosition->setCurrentIndex(1);
     }
     else
     {
-        if (pView->m_pNumberPosCh1)
-            pView->m_pNumberPosCh1->setRemain(false);
-        if (pView->m_pNumberPosCh2)
-            pView->m_pNumberPosCh2->setRemain(false);
+        pView->slotSetDurationRemaining(false);
         ComboBoxPosition->setCurrentIndex(0);
     }
     connect(ComboBoxPosition,   SIGNAL(activated(int)), this, SLOT(slotSetPositionDisplay(int)));
@@ -95,14 +89,16 @@ DlgPrefControls::DlgPrefControls(QWidget * parent, MixxxView * pView, MixxxApp *
     //
     // Rate buttons configuration
     //
+    //NOTE: THESE DEFAULTS ARE A LIE! You'll need to hack the same values into the static variables
+    //      at the top of enginebuffer.cpp
     if (m_pConfig->getValueString(ConfigKey("[Controls]","RateTempLeft")).length() == 0)
         m_pConfig->set(ConfigKey("[Controls]","RateTempLeft"),ConfigValue(QString("4.0")));
     if (m_pConfig->getValueString(ConfigKey("[Controls]","RateTempRight")).length() == 0)
         m_pConfig->set(ConfigKey("[Controls]","RateTempRight"),ConfigValue(QString("2.0")));
     if (m_pConfig->getValueString(ConfigKey("[Controls]","RatePermLeft")).length() == 0)
-        m_pConfig->set(ConfigKey("[Controls]","RatePermLeft"),ConfigValue(QString("1.0")));
+        m_pConfig->set(ConfigKey("[Controls]","RatePermLeft"),ConfigValue(QString("0.50")));
     if (m_pConfig->getValueString(ConfigKey("[Controls]","RatePermRight")).length() == 0)
-        m_pConfig->set(ConfigKey("[Controls]","RatePermRight"),ConfigValue(QString("1.0")));
+        m_pConfig->set(ConfigKey("[Controls]","RatePermRight"),ConfigValue(QString("0.05")));
 
     connect(spinBoxTempRateLeft, SIGNAL(valueChanged(double)), this, SLOT(slotSetRateTempLeft(double)));
     connect(spinBoxTempRateRight, SIGNAL(valueChanged(double)), this, SLOT(slotSetRateTempRight(double)));
@@ -114,6 +110,8 @@ DlgPrefControls::DlgPrefControls(QWidget * parent, MixxxView * pView, MixxxApp *
     spinBoxPermRateLeft->setValue(m_pConfig->getValueString(ConfigKey("[Controls]","RatePermLeft")).toDouble());
     spinBoxPermRateRight->setValue(m_pConfig->getValueString(ConfigKey("[Controls]","RatePermRight")).toDouble());
 
+    SliderRateRampSensitivity->setEnabled(true);
+    SpinBoxRateRampSensitivity->setEnabled(true);
 
     //
     // Visuals
@@ -124,8 +122,8 @@ DlgPrefControls::DlgPrefControls(QWidget * parent, MixxxView * pView, MixxxApp *
         m_pConfig->set(ConfigKey("[Controls]","Visuals"), ConfigValue(0));
 
     // Update combo box
-    ComboBoxVisuals->addItem("Waveform");
-    ComboBoxVisuals->addItem("Simple");
+    ComboBoxVisuals->addItem("On");
+    ComboBoxVisuals->addItem("Off");
     ComboBoxVisuals->setCurrentIndex(m_pConfig->getValueString(ConfigKey("[Controls]","Visuals")).toInt());
 
     connect(ComboBoxVisuals,   SIGNAL(activated(int)), this, SLOT(slotSetVisuals(int)));
@@ -138,14 +136,14 @@ DlgPrefControls::DlgPrefControls(QWidget * parent, MixxxView * pView, MixxxApp *
     QString qSkinPath(pConfig->getValueString(ConfigKey("[Config]","Path")));
     QDir dir(qSkinPath.append("skins/"));
     dir.setFilter(QDir::Dirs);
-    
+
     //
     // Default Cue Behavior
     //
-    
+
     m_pControlCueDefault1 = new ControlObjectThreadMain(ControlObject::getControl(ConfigKey("[Channel1]","cue_mode")));
     m_pControlCueDefault2 = new ControlObjectThreadMain(ControlObject::getControl(ConfigKey("[Channel2]","cue_mode")));
-    
+
     // Set default value in config file and control objects, if not present
     QString cueDefault = m_pConfig->getValueString(ConfigKey("[Controls]","CueDefault"));
     if(cueDefault.length() == 0) {
@@ -156,12 +154,12 @@ DlgPrefControls::DlgPrefControls(QWidget * parent, MixxxView * pView, MixxxApp *
 
     m_pControlCueDefault1->slotSet(cueDefaultValue);
     m_pControlCueDefault2->slotSet(cueDefaultValue);
-    
+
     // Update combo box
     ComboBoxCueDefault->addItem("CDJ Mode");
     ComboBoxCueDefault->addItem("Simple");
     ComboBoxCueDefault->setCurrentIndex(m_pConfig->getValueString(ConfigKey("[Controls]","CueDefault")).toInt());
-   
+
     connect(ComboBoxCueDefault,   SIGNAL(activated(int)), this, SLOT(slotSetCueDefault(int)));
 
     //Cue recall
@@ -228,19 +226,6 @@ DlgPrefControls::DlgPrefControls(QWidget * parent, MixxxView * pView, MixxxApp *
     slotUpdateSchemes();
 
     connect(ComboBoxSchemeconf, SIGNAL(activated(int)), this, SLOT(slotSetScheme(int)));
-    //
-    // Scale BPM configuration
-    //
-    // Set default value in config file, if not present
-    if (m_pConfig->getValueString(ConfigKey("[Controls]","ScaleBpm")).length() == 0)
-        m_pConfig->set(ConfigKey("[Controls]","ScaleBpm"), ConfigValue(1));
-
-    // Update combo box
-    ComboBoxScaleBpm->setCurrentIndex((m_pConfig->getValueString(ConfigKey("[Controls]","ScaleBpm")).toInt()+1)%2);
-
-    connect(ComboBoxScaleBpm,   SIGNAL(activated(int)), this, SLOT(slotSetScaleBpm(int)));
-    slotSetScaleBpm(0);
-
 
     //
     // Tooltip configuration
@@ -252,6 +237,22 @@ DlgPrefControls::DlgPrefControls(QWidget * parent, MixxxView * pView, MixxxApp *
     // Update combo box
     ComboBoxTooltips->setCurrentIndex((m_pConfig->getValueString(ConfigKey("[Controls]","Tooltips")).toInt()+1)%2);
 
+    //
+    // Ramping Temporary Rate Change configuration
+    //
+
+    // Set Ramp Rate On or Off
+    connect(groupBoxRateRamp, SIGNAL(toggled(bool)), this, SLOT(slotSetRateRamp(bool)));
+    groupBoxRateRamp->setChecked((bool)
+            m_pConfig->getValueString(ConfigKey("[Controls]","RateRamp")).toInt()
+    );
+
+    // Update Ramp Rate Sensitivity
+    connect(SliderRateRampSensitivity, SIGNAL(valueChanged(int)), this, SLOT(slotSetRateRampSensitivity(int)));
+    SliderRateRampSensitivity->setValue(
+        m_pConfig->getValueString(ConfigKey("[Controls]","RateRampSensitivity")).toInt()
+    );
+
     connect(ComboBoxTooltips,   SIGNAL(activated(int)), this, SLOT(slotSetTooltips(int)));
 
     slotUpdateSchemes();
@@ -277,7 +278,7 @@ void DlgPrefControls::slotUpdateSchemes()
         ComboBoxSchemeconf->setEnabled(true);
         for (int i = 0; i < schlist.size(); i++) {
             ComboBoxSchemeconf->addItem(schlist[i]);
-            
+
             if (schlist[i] == m_pConfig->getValueString(ConfigKey("[Config]","Scheme"))) {
                 ComboBoxSchemeconf->setCurrentIndex(i);
             }
@@ -344,10 +345,7 @@ void DlgPrefControls::slotSetRateDir(int)
 void DlgPrefControls::slotSetVisuals(int)
 {
     m_pConfig->set(ConfigKey("[Controls]","Visuals"), ConfigValue(ComboBoxVisuals->currentIndex()));
-    QMessageBox::information(this, tr("Information"), //make the fact that you have to restart mixxx more obvious
-    //textLabel->setText(
-      tr("Restart Mixxx before the change of visuals will take effect.")
-      );
+    m_mixxx->rebootMixxxView();
 }
 
 void DlgPrefControls::slotSetCueDefault(int)
@@ -362,15 +360,6 @@ void DlgPrefControls::slotSetCueRecall(int)
     m_pConfig->set(ConfigKey("[Controls]","CueRecall"), ConfigValue(ComboBoxCueRecall->currentIndex()));
 }
 
-void DlgPrefControls::slotSetScaleBpm(int)
-{
-    m_pConfig->set(ConfigKey("[Controls]","ScaleBpm"), ConfigValue((ComboBoxScaleBpm->currentIndex()+1)%2));
-    if (ComboBoxScaleBpm->currentIndex()==0)
-        WNumberBpm::setScaleBpm(true);
-    else
-        WNumberBpm::setScaleBpm(false);
-}
-
 void DlgPrefControls::slotSetTooltips(int)
 {
 // setGloballyEnabled currently not implemented in QT4
@@ -382,11 +371,11 @@ void DlgPrefControls::slotSetTooltips(int)
     //the eventFilter is. The value of the ConfigObject is cached at startup because it's too slow
     //to refresh it during each Tooltip event (I think), which is why we require a restart.
 
-    
+
     QMessageBox::information(this, tr("Information"), //make the fact that you have to restart mixxx more obvious
     //textLabel->setText(
       tr("Mixxx must be restarted before the changes will take effect."));
-    
+
 
 //    if (ComboBoxTooltips->currentIndex()==0)
 //        QToolTip::setGloballyEnabled(true);
@@ -411,53 +400,65 @@ void DlgPrefControls::slotSetSkin(int)
 void DlgPrefControls::slotSetPositionDisplay(int)
 {
     m_pConfig->set(ConfigKey("[Controls]","PositionDisplay"), ConfigValue(ComboBoxPosition->currentIndex()));
-
-    if (ComboBoxPosition->currentIndex()==1)
-    {
-        if (m_pView->m_pNumberPosCh1)
-            m_pView->m_pNumberPosCh1->setRemain(true);
-        if (m_pView->m_pNumberPosCh2)
-            m_pView->m_pNumberPosCh2->setRemain(true);
-    }
-    else
-    {
-        if (m_pView->m_pNumberPosCh1)
-            m_pView->m_pNumberPosCh1->setRemain(false);
-        if (m_pView->m_pNumberPosCh2)
-            m_pView->m_pNumberPosCh2->setRemain(false);
-    }
+    m_pView->slotSetDurationRemaining(ComboBoxPosition->currentIndex() == 1);
 }
 
 void DlgPrefControls::slotSetRateTempLeft(double v)
 {
     QString str;
-    str = str.setNum(v, 'f', 1);
+    str = str.setNum(v, 'f');
     m_pConfig->set(ConfigKey("[Controls]","RateTempLeft"),ConfigValue(str));
-    EngineBuffer::setTemp(v);
+    RateControl::setTemp(v);
 }
 
 void DlgPrefControls::slotSetRateTempRight(double v)
 {
     QString str;
-    str = str.setNum(v, 'f', 1);
+    str = str.setNum(v, 'f');
     m_pConfig->set(ConfigKey("[Controls]","RateTempRight"),ConfigValue(str));
-    EngineBuffer::setTempSmall(v);
+    RateControl::setTempSmall(v);
 }
 
 void DlgPrefControls::slotSetRatePermLeft(double v)
 {
     QString str;
-    str = str.setNum(v, 'f', 1);
+    str = str.setNum(v, 'f');
     m_pConfig->set(ConfigKey("[Controls]","RatePermLeft"),ConfigValue(str));
-    EngineBuffer::setPerm(v);
+    RateControl::setPerm(v);
 }
 
 void DlgPrefControls::slotSetRatePermRight(double v)
 {
     QString str;
-    str = str.setNum(v, 'f', 1);
+    str = str.setNum(v, 'f');
     m_pConfig->set(ConfigKey("[Controls]","RatePermRight"),ConfigValue(str));
-    EngineBuffer::setPermSmall(v);
+    RateControl::setPermSmall(v);
+}
+
+void DlgPrefControls::slotSetRateRampSensitivity(int sense)
+{
+    m_pConfig->set(ConfigKey("[Controls]","RateRampSensitivity"),
+                        ConfigValue(SliderRateRampSensitivity->value()));
+    RateControl::setRateRampSensitivity(sense);
+}
+
+void DlgPrefControls::slotSetRateRamp(bool mode)
+{
+    m_pConfig->set(ConfigKey("[Controls]", "RateRamp"),
+                        ConfigValue(groupBoxRateRamp->isChecked()));
+    RateControl::setRateRamp(mode);
+
+    /*
+    if ( mode )
+    {
+        SliderRateRampSensitivity->setEnabled(TRUE);
+        SpinBoxRateRampSensitivity->setEnabled(TRUE);
+    }
+    else
+    {
+        SliderRateRampSensitivity->setEnabled(FALSE);
+        SpinBoxRateRampSensitivity->setEnabled(FALSE);
+    }*/
 }
 
 void DlgPrefControls::slotApply()
@@ -474,5 +475,6 @@ void DlgPrefControls::slotApply()
         m_pConfig->set(ConfigKey("[Controls]","RateDir"), ConfigValue(0));
     else
         m_pConfig->set(ConfigKey("[Controls]","RateDir"), ConfigValue(1));
+
 }
 
