@@ -112,6 +112,7 @@ void DlgPrefEQ::slotUpdateHiEQ()
         SliderHiEQ->setValue( SliderLoEQ->value());
     }
 	m_highEqFreq = getEqFreq(SliderHiEQ->value());
+    validate_levels();
 	if(m_highEqFreq < 1000)
 		TextHiEQ->setText( QString("%1 Hz").arg(m_highEqFreq));
 	else
@@ -128,6 +129,7 @@ void DlgPrefEQ::slotUpdateLoEQ()
         SliderLoEQ->setValue( SliderHiEQ->value());
     }
 	m_lowEqFreq = getEqFreq(SliderLoEQ->value());
+    validate_levels();
     if(m_lowEqFreq < 1000)
 		TextLoEQ->setText( QString("%1 Hz").arg(m_lowEqFreq));
 	else
@@ -175,3 +177,15 @@ int DlgPrefEQ::getEqFreq(int sliderVal)
 	}
 }
 
+void DlgPrefEQ::validate_levels() {
+    if (m_lowEqFreq == m_highEqFreq) {
+        // magic numbers: 16 is the low, 20050 is the high
+        if (m_lowEqFreq == 16) {
+            ++m_highEqFreq;
+        } else if (m_highEqFreq == 20050) {
+            --m_lowEqFreq;
+        } else {
+            ++m_highEqFreq;
+        }
+    }
+}
