@@ -30,12 +30,23 @@
 #include "sounddevice.h"
 #include "soundmanager.h"
 #include <qwidget.h>
+#include <math.h>
 
 #ifdef __C_METRICS__
 #include <cmetrics.h>
 #include "defs_mixxxcmetrics.h"
 #endif
 
+// Calculates log2 of number.  
+// 
+#ifndef log2
+double log2( double n )  
+{  
+    // log(n)/log(2) is log2.  
+    return log( n ) / log( 2.0 );   // MSVC doesn't know how to take logs of ints.
+}
+#endif
+
 DlgPrefSound::DlgPrefSound(QWidget * parent, SoundManager * _soundman,
                            ConfigObject<ConfigValue> * _config) : QWidget(parent), Ui::DlgPrefSoundDlg()
 {
@@ -227,27 +238,36 @@ void DlgPrefSound::slotLatency()
     TextLabelLatency->setText(QString("%1 ms").arg(getSliderLatencyMsec(SliderLatency->value())));
 }
 
+/** Converts a slider tick position into a latency in milliseconds */
 int DlgPrefSound::getSliderLatencyMsec(int val)
 {
-    if (val>16)
-        val = (val-12)*(val-12);
+    qDebug() << "getSliderLatencyMsec in: " << val;
+    int sampleRate = config->getValueString(ConfigKey("[Soundcard]","Samplerate")).toInt();
+    val = ((float)pow(2, (float)val) / sampleRate) * 1000;
+    qDebug() << "getSliderLatencyMsec out: " << val;
     return val;
 }
 
+/** Converts a latency in milliseconds into a slider tick position */
 int DlgPrefSound::getSliderLatencyVal(int val)
 {
-    if (val<=16)
-        return val;
+    //The input param "val" is in MSec, and we want to convert to a slider tick position.
+    int sampleRate = config->getValueString(ConfigKey("[Soundcard]","Samplerate")).toInt();
+    int iFramesPerBuffer = ((float)val/1000.0f) * sampleRate;
 
-    int i=5;
-    while (i*i<val)
-        i++;
-    return 12+i;
+    //Round to the nearest power-of-two.
+    unsigned int i;
+    iFramesPerBuffer &= INT_MAX;
+    for (i = 1; iFramesPerBuffer > i; i <<= 1) ;
+    iFramesPerBuffer = i;
+
+    int exponent = log2(iFramesPerBuffer);
+
+    return exponent;
 }
 
 void DlgPrefSound::slotApply()
 {
-	m_configError = false;
     qDebug() << "DlgPrefSound::Apply";
 
     // Update the config object with parameters from dialog
@@ -292,23 +312,18 @@ void DlgPrefSound::slotApply()
     m_pSoundManager->closeDevices();
 
     // Not much to do if the API is None...
-	try{
-		if (config->getValueString(ConfigKey("[Soundcard]","SoundApi"))!="None")
-		{
-			if (m_pSoundManager->setupDevices() != 0)
-				QMessageBox::warning(0, "Configuration error","Audio device could not be opened");
-			else
-				slotUpdate();
-		}
-	}
-	catch(int e)
+    int deviceOpenError = 0;
+	if (config->getValueString(ConfigKey("[Soundcard]","SoundApi"))!="None")
 	{
-		if (e == MIXXX_DUPLICATE_OUTPUT_CHANNEL_EXCEPTION)
-		    QMessageBox::warning(0, "Configuration error", "You cannot send multiple outputs to a single channel");
-		if (e == MIXXX_DUPLICATE_INPUT_CHANNEL_EXCEPTION)
-		    QMessageBox::warning(0, "Configuration error", "You cannot use a single pair of channels for both decks");		
-		m_parent->setHidden(false);
-		m_configError = true;
+	    deviceOpenError = m_pSoundManager->setupDevices();
+	    if (deviceOpenError == MIXXX_ERROR_DUPLICATE_OUTPUT_CHANNEL)
+	        QMessageBox::warning(0, "Configuration error", "You cannot send multiple outputs to a single channel");
+	    else if (deviceOpenError == MIXXX_ERROR_DUPLICATE_INPUT_CHANNEL)
+	        QMessageBox::warning(0, "Configuration error", "You cannot use a single pair of channels for both decks");		
+	    else if (deviceOpenError != 0)
+		    QMessageBox::warning(0, "Configuration error","Audio device could not be opened");
+		else
+			slotUpdate();
 	}
 	
 #ifdef __C_METRICS__
