@@ -5,6 +5,10 @@
 /** MidiMessage constructor */
 MidiMessage::MidiMessage(MidiStatusByte status, int midino, char midichannel)
 {
+    //Register this class with QT so we can use this bad boy in signals/slots.
+    qRegisterMetaType<MidiMessage>("MidiMessage");
+    qRegisterMetaType<MidiStatusByte>("MidiStatusByte");
+
     m_midiStatusByte = status;
     m_midiNo = midino;
     m_midiStatusByte |= midichannel; //Jam midichannel into low nibble of status byte.
