@@ -199,10 +199,10 @@ MixxxApp::MixxxApp(QApplication * a, struct CmdlineArgs args)
     QDir dir(config->getValueString(ConfigKey("[Playlist]","Directory")));
     if ((config->getValueString(ConfigKey("[Playlist]","Directory")).length()<1) || (!dir.exists()))
     {
-        QString fd = QFileDialog::getExistingDirectory(this, 
+        QString fd = QFileDialog::getExistingDirectory(this,
                          tr("Choose music library directory"),
                          QDesktopServices::storageLocation(QDesktopServices::MusicLocation));
-        
+
         if (fd != "")
         {
             config->set(ConfigKey("[Playlist]","Directory"), fd);
@@ -227,15 +227,6 @@ MixxxApp::MixxxApp(QApplication * a, struct CmdlineArgs args)
     m_pPlayer1 = new Player(config, buffer1, "[Channel1]");
     m_pPlayer2 = new Player(config, buffer2, "[Channel2]");
 
-    //Connect the player to the track collection so that when a track is unloaded,
-    //it's data (eg. waveform summary) is saved back to the database.
-    connect(m_pPlayer1, SIGNAL(unloadingTrack(TrackPointer)),
-            &(m_pLibrary->getTrackCollection()->getTrackDAO()),
-            SLOT(saveTrack(TrackPointer)));
-    connect(m_pPlayer2, SIGNAL(unloadingTrack(TrackPointer)),
-            &(m_pLibrary->getTrackCollection()->getTrackDAO()),
-            SLOT(saveTrack(TrackPointer)));
-
     view=new MixxxView(frame, kbdconfig, qSkinPath, config, m_pPlayer1, m_pPlayer2,
                        m_pLibrary);
 
@@ -332,6 +323,10 @@ MixxxApp::MixxxApp(QApplication * a, struct CmdlineArgs args)
     ControlObject::getControl(ConfigKey("[Channel1]","TrackEndMode"))->queueFromThread(config->getValueString(ConfigKey("[Controls]","TrackEndModeCh1")).toDouble());
     ControlObject::getControl(ConfigKey("[Channel2]","TrackEndMode"))->queueFromThread(config->getValueString(ConfigKey("[Controls]","TrackEndModeCh2")).toDouble());
 
+
+    qRegisterMetaType<MidiMessage>("MidiMessage");
+    qRegisterMetaType<MidiStatusByte>("MidiStatusByte");
+
     // Initialise midi
     m_pMidiDeviceManager = new MidiDeviceManager(config);
     //TODO: Try to open MIDI devices?
@@ -600,7 +595,11 @@ void MixxxApp::initActions()
     optionsBeatMark = new QAction(tr("&Audio Beat Marks"), this);
 
     optionsFullScreen = new QAction(tr("&Full Screen"), this);
+#ifdef __APPLE__
+    optionsFullScreen->setShortcut(tr("Ctrl+F"));
+#else
     optionsFullScreen->setShortcut(tr("F11"));
+#endif
     optionsFullScreen->setShortcutContext(Qt::ApplicationShortcut);
     //QShortcut * shortcut = new QShortcut(QKeySequence(tr("Esc")),  this);
     //    connect(shortcut, SIGNAL(activated()), this, SLOT(slotQuitFullScreen()));
@@ -1147,6 +1146,7 @@ void MixxxApp::slotHelpAbout()
 "Antonio Passamani<br>"
 "Guy Martin<br>"
 "Anders Gunnarson<br>"
+"Mikko Jania<br>"
 
 "</p>"
 "<p align=\"center\"><b>And special thanks to:</b></p>"
