@@ -18,71 +18,76 @@
 #ifndef TRACKINFOOBJECT_H
 #define TRACKINFOOBJECT_H
 
-#include <qobject.h>
+#include <QList>
+#include <QObject>
+#include <QFileInfo>
 #include <q3memarray.h>
 #include <q3valuelist.h>
-#include <qmutex.h>
+#include <QMutex>
 #include <QVector>
+#include <QSharedPointer>
+#include <QWeakPointer>
 
 #include "defs.h"
 
+#include "library/dao/cue.h"
+
 class QString;
 class QDomElement;
 class QDomDocument;
 class QDomNode;
-//class WTrackTable;
-//class WTrackTableItem;
-class WOverview;
 class ControlObject;
-class BpmDetector;
-class BpmReceiver;
-class BpmScheme;
 class TrackPlaylist;
+class Cue;
+
+class TrackInfoObject;
+
+typedef QSharedPointer<TrackInfoObject> TrackPointer;
+typedef QWeakPointer<TrackInfoObject> TrackWeakPointer;
 
-//template <class T> class Segmentation;
 #include "segmentation.h"
 
-#define NumBpmFactors 4
-
-const float Factors[NumBpmFactors]={0.33, 0.5, 2, 0.25};
-
 class TrackInfoObject : public QObject
 {
     Q_OBJECT
 public:
     /** Initialize a new track with the filename. */
-    TrackInfoObject(const QString sPath, const QString sFile);
+    TrackInfoObject(const QString sLocation="", bool parseHeader=true);
+    // Initialize track with a QFileInfo class
+    TrackInfoObject(QFileInfo& fileInfo, bool parseHeader=true);
     /** Creates a new track given information from the xml file. */
     TrackInfoObject(const QDomNode &);
-    ~TrackInfoObject();
+    virtual ~TrackInfoObject();
+
     /** Returns true if the object contains valid information */
     bool isValid() const;
     int parse();
-    /** Checks if the file given in m_sFilename really exists on the disc, and
-        updates the m_bExists flag accordingly. Returns true if the file
-        exists */
-    bool checkFileExists();
     void writeToXML( QDomDocument &, QDomElement & );
-    /** Insert at the values in a WTrackTable at a given row */
-    //void insertInTrackTableRow(WTrackTable *pTableTrack, int iRow);
-    /** Reset pointers to table cells */
-    //void removeFromTrackTable();
-	/** Assists in clearing the table*/
-	//void clearTrackTableRow();
+
     /** Returns the duration in seconds */
     int getDuration() const;
     /** Returns the duration as a string: H:MM:SS */
     QString getDurationStr() const;
-    /** Returns the location of the file, included path */
+
+    // Accessors for various stats of the file on disk. These are auto-populated
+    // when the TIO is constructed, or when setLocation() is called.
+
+    // Returns absolute path to the file, including the filename.
     QString getLocation() const;
+    // Returns the absolute path to the directory containing the file
+    QString getDirectory() const;
+    // Returns the filename of the file.
+    QString getFilename() const;
+    // Returns the length of the file in bytes
+    int getLength() const;
+    // Returns whether the file exists on disk or not. Updated as of the time
+    // the TrackInfoObject is created, or when setLocation() is called.
+    bool exists() const;
+
     /** Returns BPM */
     float getBpm() const;
     /** Set BPM */
     void setBpm(float);
-    /** Set BPM Correction Factors */
-    void generateBpmFactors();
-    /** Get BPM Correction Factors */
-    void getBpmFactors(float*) const;
     /** Returns BPM as a string */
     QString getBpmStr() const;
     /** Retruns if BPM was confirmed (edited or verified manually) */
@@ -109,8 +114,6 @@ public:
     void setBeatFirst(float);
     /** Get first beat pos */
     float getBeatFirst() const;
-    /** Retruns the length of the file in bytes */
-    int getLength() const;
     /** Set sample rate */
     void setSampleRate(int iSampleRate);
     /** Get sample rate */
@@ -123,6 +126,8 @@ public:
     QString getInfo() const;
     /** Set duration in seconds */
     void setDuration(int);
+
+    /** Getter/Setter methods for metadata */
     /** Return title */
     QString getTitle() const;
     /** Set title */
@@ -131,26 +136,30 @@ public:
     QString getArtist() const;
     /** Set artist */
     void setArtist(QString);
-    /** Return filename */
-    QString getFilename() const;
-    /** Return true if the file exist */
-    bool exists()  const;
+    /** Return album */
+    QString getAlbum() const;
+    /** Set album */
+    void setAlbum(QString);
+    /** Return Year */
+    QString getYear() const;
+    /** Set year */
+    void setYear(QString);
+    /** Return genre */
+    QString getGenre() const;
+    /** Set genre */
+    void setGenre(QString);
+    /** Return Track Number */
+    QString getTrackNumber() const;
+    /** Set Track Number */
+    void setTrackNumber(QString);
     /** Return number of times the track has been played */
     int getTimesPlayed() const;
     /** Increment times played with one */
     void incTimesPlayed();
-    /** Sets the filepath */
-    void setFilepath(QString);
-    /** Returns the score */
-    int getScore() const;
-    /** Returns the score as string */
-    QString getScoreStr() const;
-    /** Updates the score */
-    void updateScore();
-    /** Get id */
+
     int getId() const;
-    /** Set id */
-    void setId(int iId);
+
+
     /** Get URL for track */
     QString getURL();
     /** Set URL for track */
@@ -165,47 +174,104 @@ public:
     double getVisualResampleRate();
 
     /** Set pointer to waveform summary -- updates UI by default */
-    void setWaveSummary(Q3MemArray<char> *pWave, Q3ValueList<long> *pSegmentation, bool updateUI = true);
+    void setWaveSummary(const QByteArray* pWave, bool updateUI = true);
 
     /** Returns a pointer to waveform summary */
-    Q3MemArray<char> *getWaveSummary();
-    /** Returns a pointer to segmentation summary */
-    Q3ValueList<long> *getSegmentationSummary();
-    /** Return the next track as listed in WTrackTable */
-    TrackInfoObject *getNext(TrackPlaylist *pPlaylist);
-    /** Return the previous track as listed in WTrackTable */
-    TrackInfoObject *getPrev(TrackPlaylist *pPlaylist);
-    /** Set corresponding overview widget */
-    void setOverviewWidget(WOverview *p);
+    const QByteArray* getWaveSummary();
+
     /** Set pointer to ControlObject holding BPM value in engine */
     void setBpmControlObject(ControlObject *p);
-    /** Set pointer to ControlObject holding duration value in engine */
-    void setDurationControlObject(ControlObject *p);
-	QString getFilepath() const;
+
     /** Save the cue point (in samples... I think) */
     void setCuePoint(float cue);
     /** Get saved the cue point */
     float getCuePoint();
 
-	const Segmentation<QString>* getChordData() { return &m_chordData; }
-	void setChordData(Segmentation<QString> cd) {
-		m_chordData = cd; 
-	}
+    // Calls for managing the track's cue points
+    Cue* addCue();
+    void removeCue(Cue* cue);
+    const QList<Cue*>& getCuePoints();
+    void setCuePoints(QList<Cue*> cuePoints);
 
-private:
-    /** Method for parsing information from knowing only the file name.
-        It assumes that the filename is written like: "artist - trackname.xxx" */
+    bool isDirty();
+
+    // Signals to the creator of this TrackInfoObject to save the Track as it
+    // may be deleted.
+    void doSave();
+
+    // Returns true if the track location has changed
+    bool locationChanged();
+
+    /** Set the track's full file path */
+    void setLocation(QString location);
+
+    const Segmentation<QString>* getChordData();
+    void setChordData(Segmentation<QString> cd);
+
+  public slots:
+    void slotCueUpdated();
+
+  signals:
+    void wavesummaryUpdated(TrackInfoObject*);
+    void bpmUpdated(double bpm);
+    void cuesUpdated();
+    void changed();
+    void dirty();
+    void clean();
+    void save();
+
+  private:
+
+    // Common initialization function between all TIO constructors.
+    void initialize(bool parseHeader);
+
+    // Initialize all the location variables.
+    void populateLocation(QFileInfo& fileInfo);
+
+    // Method for parsing information from knowing only the file name.  It
+    // assumes that the filename is written like: "artist - trackname.xxx"
     void parseFilename();
-    /** Flag which determines if the file exists or not. */
-    bool m_bExists;
-    /** File name */
+
+    // Set whether the TIO is dirty not. This should never be called except by
+    // TIO local methods or the TrackDAO.
+    void setDirty(bool bDirty);
+
+    // Set a unique identifier for the track. Only used by services like
+    // TrackDAO
+    void setId(int iId);
+
+    // Flag that indicates whether or not the TIO has changed. This is used by
+    // TrackDAO to determine whether or not to write the Track back.
+    bool m_bDirty;
+
+    // Special flag for telling if the track location was changed.
+    bool m_bLocationChanged;
+
+    // The filename
     QString m_sFilename;
-    /** Path for the track file */
-    QString m_sFilepath;
+    // The full path to the file, including the filename.
+    QString m_sLocation;
+    // The full path to the directory containing the file.
+    QString m_sDirectory;
+    // Length of track in bytes
+    int m_iLength;
+    // Flag which indicates whether the file exists or not.
+    bool m_bExists;
+
+    /** Metadata */
+    /** Album */
+    QString m_sAlbum;
     /** Artist */
     QString m_sArtist;
     /** Title */
     QString m_sTitle;
+    /** Genre */
+    QString m_sGenre;
+    /** Year */
+    QString m_sYear;
+    /** Track Number */
+    QString m_sTrackNumber;
+
     /** File type */
     QString m_sType;
     /** User comment */
@@ -214,20 +280,16 @@ private:
     QString m_sURL;
     /** Duration of track in seconds */
     int m_iDuration;
-    /** Length of track in bytes */
-    int m_iLength;
     /** Sample rate */
     int m_iSampleRate;
     /** Number of channels */
     int m_iChannels;
-    /** Bitrate */
+    /** Bitrate, number of kilobits per second of audio in the track*/
     int m_iBitrate;
     /** Number of times the track has been played */
     int m_iTimesPlayed;
     /** Beat per minutes (BPM) */
     float m_fBpm;
-    /** Bpm Correction Factors */
-    float* m_fBpmFactors;
     /** Minimum BPM range. If this is 0.0, then the config min BPM will be used */
     float m_fMinBpm;
     /** Maximum BPM range. If this is 0.0, then the config max BPM will be used */
@@ -238,42 +300,29 @@ private:
     bool m_bHeaderParsed;
     /** Position of first beat in song */
     float m_fBeatFirst;
-    /** Score. Reflects the relative number of times the track has been played */
-    int m_iScore;
     /** Id. Unique ID of track */
     int m_iId;
     /** Cue point in samples or something */
     float m_fCuePoint;
 
+    // The list of cue points for the track
+    QList<Cue*> m_cuePoints;
+
     /** Pointer to visual waveform info */
     QVector<float> *m_pVisualWave;
-    /** Pointer to summary wave info */
-    Q3MemArray<char> *m_pWave;
-    /** Pointer to summary segmentation */
-    Q3ValueList<long> *m_pSegmentation;
-    /** WTrackTableItems are representations of the values actually shown in the WTrackTable */
-    //WTrackTableItem *m_pTableItemScore, *m_pTableItemTitle, *m_pTableItemArtist, *m_pTableItemComment, *m_pTableItemType,
-    //                *m_pTableItemDuration, *m_pTableItemBpm, *m_pTableItemBitrate;
-    /** Pointer to WTrackTable where the current item was inserted last */
-    //WTrackTable *m_pTableTrack;
-
-    /** Maximum number of times any one track have been played */
-    static int siMaxTimesPlayed;
+    /** Wave summary info */
+    QByteArray m_waveSummary;
 
     /** Mutex protecting access to object */
     mutable QMutex m_qMutex;
-    /** Corresponding WOverview widget */
-    WOverview *m_pOverviewWidget;
+
     /** True if object contains valid information */
     bool m_bIsValid;
-    /** Pointer to ControlObject of BPM value (only set when the track is loaded in a player) */
-    ControlObject *m_pControlObjectBpm;
-    /** Pointer to ControlObject of duration value (only set when the track is loaded in a player) */
-    ControlObject *m_pControlObjectDuration;
-    int iTemp;
-    double m_dVisualResampleRate;
 
-	Segmentation<QString> m_chordData;
+    double m_dVisualResampleRate;
+    Segmentation<QString> m_chordData;
+
+    friend class TrackDAO;
 };
 
 #endif
