@@ -4,6 +4,7 @@
 
 #include <QObject>
 #include <QColor>
+#include <QPixmap>
 #include <QVector>
 
 class QDomNode;
@@ -11,38 +12,52 @@ class QPainter;
 class QPaintEvent;
 
 #include "configobject.h"
-#include "renderobject.h"
+#include "waveform/renderobject.h"
 
 class ConfigKey;
 class ControlObjectThreadMain;
 class WaveformRenderer;
-class TrackInfoObject;
 
 class WaveformRenderMark : public RenderObject {
     Q_OBJECT
 public:
-    WaveformRenderMark(const char *group, ConfigKey key, WaveformRenderer *parent);
+    explicit WaveformRenderMark(const char* pGroup,
+                                WaveformRenderer *parent);
     void resize(int w, int h);
     void setup(QDomNode node);
-    void draw(QPainter *pPainter, QPaintEvent *event, QVector<float> *buffer, double playPos, double rateAdjust);
-    void newTrack(TrackInfoObject *pTrack);
+    void draw(QPainter *pPainter, QPaintEvent *event,
+              QVector<float> *buffer, double playPos, double rateAdjust);
+    void newTrack(TrackPointer pTrack);
 
 public slots:
     void slotUpdateMarkPoint(double mark);
     void slotUpdateTrackSamples(double samples);
 private:
+    void setupMarkPixmap();
+
+    enum MarkAlign {
+        TOP = 0,
+        BOTTOM,
+        CENTER
+    };
+
+    const char* m_pGroup;
     WaveformRenderer *m_pParent;
     ControlObjectThreadMain *m_pMarkPoint;
     ControlObjectThreadMain *m_pTrackSamples;
-    TrackInfoObject *m_pTrack;
+    TrackPointer m_pTrack;
 
     int m_iMarkPoint;
     int m_iWidth, m_iHeight;
-    QColor markColor;
+    QColor m_markColor;
+    QColor m_textColor;
+    QString m_markText;
+    QString m_markPixmapPath;
+    MarkAlign m_markAlign;
+    QPixmap m_markPixmap;
+    bool m_bHasCustomPixmap;
     double m_dSamplesPerDownsample;
 
-    ConfigKey m_key;
-
     int m_iNumSamples;
     int m_iSampleRate;
 };
